from anyblok_pyramid.tests.testcase import PyramidDBTestCase
from anyblok import Declarations
from pyramid.response import Response
register = Declarations.register
PyramidHTTP = Declarations.PyramidHTTP
PyramidMixin = Declarations.PyramidMixin
PyramidException = Declarations.Exception.PyramidException
Core = Declarations.Core


class TestDeclarationPyramidHTTP(PyramidDBTestCase):

    def check_controller(self):
        res = self.http('/test/methodA', params={'a': 2, 'b': 3})
        self.assertEqual(eval(res.body), {'a': 4, 'b': 6})
        res = self.http('/test/methodB', params={'a': 2, 'b': 3})
        self.assertEqual(eval(res.body), {'a': 6, 'b': 9})

    def add_routes(self):
        PyramidHTTP.add_route('methodA', '/test/methodA')
        PyramidHTTP.add_route('method_B', '/test/methodB')

    def add_routes2(self):
        PyramidHTTP.add_route('test2methodA', '/test2/methodA')
        PyramidHTTP.add_route('test2methodB', '/test2/methodB')

    def test_http(self):
        def add_http_contoller():

            @register(PyramidHTTP)
            class Test:

                @PyramidHTTP.view()
                def methodA(self, *args, **kwargs):
                    return Response(str({x: int(y) * 2
                                         for x, y in kwargs.items()}))

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_http_without_routes(self):
        def add_http_contoller():

            @register(PyramidHTTP)
            class Test:

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str({x: int(y) * 2
                                         for x, y in kwargs.items()}))

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

        with self.assertRaises(Declarations.Exception.PyramidException):
            self.init_registry(add_http_contoller)

    def test_authentificated(self):
        def add_http_contoller():

            @register(PyramidHTTP)
            class Test:

                @PyramidHTTP.view()
                @PyramidHTTP.authentificated()
                def methodA(self, *args, **kwargs):
                    return Response(str({x: int(y) * 2
                                         for x, y in kwargs.items()}))

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_unknown_property(self):
        def add_http_contoller():

            @register(PyramidHTTP)
            class Test:

                @PyramidHTTP.view()
                @PyramidHTTP.check_properties(unknown_property=True)
                def methodA(self, *args, **kwargs):
                    return Response(str({x: int(y) * 2
                                         for x, y in kwargs.items()}))

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            self.add_routes()

        self.init_registry(add_http_contoller)
        with self.assertRaises(PyramidException.PyramidInvalidProperty):
            self.check_controller()

    def test_simple_subclass_controller(self):
        def add_http_contoller():

            @register(PyramidHTTP)
            class Test:

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str({x: int(y) * 2
                                         for x, y in kwargs.items()}))

            @register(PyramidHTTP)  # noqa
            class Test:

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_simple_subclass_base_http(self):
        def add_http_contoller():

            @register(Core)
            class PyramidBaseHTTP:

                def methodA(self, **kwargs):
                    return {x: int(y) * 2 for x, y in kwargs.items()}

            @register(PyramidHTTP)
            class Test:

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str(super(Test, self).methodA(**kwargs)))

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_mixin_one_controller(self):
        def add_http_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: int(y) * 2 for x, y in kwargs.items()}

            @register(PyramidHTTP)
            class Test(PyramidMixin.PyMixin):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str(super(Test, self).methodA(**kwargs)))

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_mixin_two_controller(self):
        def add_http_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: int(y) * 2 for x, y in kwargs.items()}

            @register(PyramidHTTP)
            class Test(PyramidMixin.PyMixin):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str(super(Test, self).methodA(**kwargs)))

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            @register(PyramidHTTP)
            class Test2(PyramidMixin.PyMixin):

                @PyramidHTTP.view(route_name='test2methodA')
                def methodA(self, **kwargs):
                    return Response(str(super(Test2, self).methodA(**kwargs)))

            self.add_routes()
            self.add_routes2()

        self.init_registry(add_http_contoller)
        self.check_controller()
        res = self.http('/test2/methodA', params={'a': 2, 'b': 3})
        self.assertEqual(eval(res.body), {'a': 4, 'b': 6})

    def test_mixin_one_controller_with_subclass(self):
        def add_http_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: int(y) * 2 for x, y in kwargs.items()}

            @register(PyramidHTTP)
            class Test(PyramidMixin.PyMixin):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str(super(Test, self).methodA(**kwargs)))

            def inherit_Test():
                # put in the function else python take only on the first
                # Test class, because the fingerprint are the same
                @register(PyramidHTTP)
                class Test:

                    @PyramidHTTP.view()
                    def method_B(self, **kwargs):
                        return Response(str({x: int(y) * 3
                                             for x, y in kwargs.items()}))

            inherit_Test()

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_mixin_one_controller_by_subclass(self):
        def add_http_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: int(y) * 2 for x, y in kwargs.items()}

            @register(PyramidHTTP)
            class Test:

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            @register(PyramidHTTP)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str(super(Test, self).methodA(**kwargs)))

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_mixin_one_controller_by_subclass_and_with(self):
        def add_http_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: int(y) * 2 for x, y in kwargs.items()}

            @register(PyramidHTTP)
            class Test:

                def method_B(self, **kwargs):
                    return {x: int(y) * 3 for x, y in kwargs.items()}

            @register(PyramidHTTP)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str(super(Test, self).methodA(**kwargs)))

            def inherit_Test():
                # put in the function else python take only on the first
                # Test class, because the fingerprint are the same
                @register(PyramidHTTP)
                class Test:

                    @PyramidHTTP.view()
                    def method_B(self, **kwargs):
                        return Response(str(
                            super(Test, self).method_B(**kwargs)))

            inherit_Test()

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_mixin_one_controller_with_subclass_and_subclass_mixin(self):
        def add_http_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return None

            @register(PyramidHTTP)
            class Test:

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            @register(PyramidHTTP)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str(super(Test, self).methodA(**kwargs)))

            @register(PyramidMixin)  # noqa
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: int(y) * 2 for x, y in kwargs.items()}

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_inherit_by_another_controller(self):
        def add_http_contoller():

            @register(PyramidHTTP)
            class Test2:

                @PyramidHTTP.view(route_name='test2methodA')
                def methodA(self, **kwargs):
                    return Response(str({x: int(y) * 2
                                         for x, y in kwargs.items()}))

                @PyramidHTTP.view(route_name='test2methodB')
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            @register(PyramidHTTP)
            class Test(PyramidHTTP.Test2):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return super(Test, self).method_B(**kwargs)

            self.add_routes()
            self.add_routes2()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_inherit_by_another_controller_and_subclass_maincontroller(self):
        def add_http_contoller():

            @register(PyramidHTTP)
            class Test2:

                @PyramidHTTP.view(route_name='test2methodB')
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            @register(PyramidHTTP)
            class Test(PyramidHTTP.Test2):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return super(Test, self).method_B(**kwargs)

            @register(PyramidHTTP)  # noqa
            class Test2:

                @PyramidHTTP.view(route_name='test2methodA')
                def methodA(self, **kwargs):
                    return Response(str({x: int(y) * 2
                                         for x, y in kwargs.items()}))

            self.add_routes()
            self.add_routes2()

        self.init_registry(add_http_contoller)
        self.check_controller()

    def test_inherit_core_and_mixin(self):
        def add_http_contoller():

            @register(Core)
            class PyramidBase:

                def methodA(self, **kwargs):
                    return {x: int(y) * 2 for x, y in kwargs.items()}

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return super(PyMixin, self).methodA(**kwargs)

            @register(PyramidHTTP)
            class Test(PyramidMixin.PyMixin):

                @PyramidHTTP.view()
                def methodA(self, **kwargs):
                    return Response(str(super(Test, self).methodA(**kwargs)))

                @PyramidHTTP.view()
                def method_B(self, **kwargs):
                    return Response(str({x: int(y) * 3
                                         for x, y in kwargs.items()}))

            self.add_routes()

        self.init_registry(add_http_contoller)
        self.check_controller()



from anyblok_pyramid._argsparse import (define_wsgi_option,
                                        define_beaker_option,
                                        define_wsgi_debug_option)
from anyblok.tests.testcase import TestCase
from anyblok.tests.test_argsparse import MockArgumentParser


class TestArgsParseOption(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TestArgsParseOption, cls).setUpClass()
        cls.parser = MockArgumentParser()
        cls.group = cls.parser.add_argument_group('label')
        cls.configuration = {}
        cls.function = {
            'define_wsgi_option': define_wsgi_option,
            'define_beaker_option': define_beaker_option,
            'define_wsgi_debug_option': define_wsgi_debug_option,
        }

    def test_define_wsgi_option(self):
        self.function['define_wsgi_option'](self.parser, self.configuration)

    def test_define_beaker_option(self):
        self.function['define_beaker_option'](self.parser, self.configuration)

    def test_define_wsgi_debug_option(self):
        self.function['define_wsgi_debug_option'](self.parser,
                                                  self.configuration)



from anyblok import Declarations
register = Declarations.register
Core = Declarations.Core
PyramidException = Declarations.Exception.PyramidException


@register(PyramidException)
class PyramidInvalidFunction(PyramidException):
    """ Sub class of Pyramid Exception """


@register(PyramidException)
class PyramidInvalidView(PyramidException):
    """ Sub class of Pyramid Exception """


@register(PyramidException)
class PyramidInvalidMethod(PyramidException):
    """ Sub class of Pyramid Exception """


@register(PyramidException)
class PyramidInvalidProperty(PyramidException):
    """ Sub class of Pyramid Exception """


@register(Core)
class PyramidBase:

    def __init__(self, request):
        self.request = request
        self.session = request.session

    def check_function(self, function):
        if not hasattr(self, function):
            raise PyramidInvalidFunction("%s has no %s function" % (
                self.__registry_name__, function))

        if function in self.properties:
            for k, v in self.properties[function].items():
                func = 'check_property_%s' % k
                if not hasattr(self, func):
                    raise PyramidInvalidProperty('%s has no %s function' % (
                        self.__registry_name__, func))

                getattr(self, func)(v)

    def check_property_authentificated(self, value):
        return True


@register(Core)
class PyramidBaseHTTP:

    def get_function_from_view(self, view):
        if view not in self.views:
            raise PyramidInvalidView('%s has no %s view' % (
                self.__registry_name__, view))

        return self.views[view]


@register(Core)
class PyramidBaseRPC:

    def get_function_from_method(self, method):
        if method not in self.rpc_methods:
            raise PyramidInvalidMethod('%s has no %s method' % (
                self.__registry_name__, method))

        return self.rpc_methods[method]



from anyblok_pyramid.tests.testcase import PyramidDBTestCase
from pyramid.response import Response
from webtest import AppError


class TestDeclarationPyramid(PyramidDBTestCase):
    def add_in_registry(self, path=None):
        from anyblok import Declarations
        Pyramid = Declarations.Pyramid

        @Pyramid.add_view('MyView')
        def my_view(request, **kwargs):
            return Response(str(kwargs))

        Pyramid.add_route('MyView', pattern=path)

    def test_add_view(self):
        path = '/test/add/view'
        self.init_registry(self.add_in_registry, path=path)
        response = self.webserver.get(path)
        self.assertEqual(response.status, '200 OK')

    def test_param(self):
        path = '/test/add/view/'
        self.init_registry(self.add_in_registry, path=path + '{name}')
        response = self.webserver.get(path + 'AnyBlok')
        self.assertEqual(response.status, '200 OK')
        self.assertEqual(eval(response.body), {'name': 'AnyBlok'})

    def test_argument(self):
        path = '/test/add/view'
        self.init_registry(self.add_in_registry, path=path)
        response = self.webserver.get(path + '?name=AnyBlok')
        self.assertEqual(response.status, '200 OK')
        self.assertEqual(eval(response.body), {'name': 'AnyBlok'})

    def add_in_registry_rest(self, path=None, request_method='GET'):
        from anyblok import Declarations
        Pyramid = Declarations.Pyramid

        @Pyramid.add_view('MyView', request_method=request_method)
        def my_view(request):
            return Response(request_method)

        Pyramid.add_route('MyView', path)

    def test_REst_get(self):
        path = '/test/rest/get'
        self.init_registry(self.add_in_registry_rest, path=path)
        response = self.webserver.get(path)
        self.assertEqual(response.status, '200 OK')
        for request_method in ('put', 'post', 'delete'):
            try:
                getattr(self.webserver, request_method)(path)
                self.fail('%r are not allowed for this path %r' % (
                    request_method, path))
            except AppError as e:
                if '404' not in e.args[0]:
                    raise

    def test_REst_put(self):
        path = '/test/rest/put'
        self.init_registry(self.add_in_registry_rest, path=path,
                           request_method='PUT')
        response = self.webserver.put(path)
        self.assertEqual(response.status, '200 OK')
        for request_method in ('get', 'post', 'delete'):
            try:
                getattr(self.webserver, request_method)(path)
                self.fail('%r are not allowed for this path %r' % (
                    request_method, path))
            except AppError as e:
                if '404' not in e.args[0]:
                    raise

    def test_REst_post(self):
        path = '/test/rest/post'
        self.init_registry(self.add_in_registry_rest, path=path,
                           request_method='POST')
        response = self.webserver.post(path)
        self.assertEqual(response.status, '200 OK')
        for request_method in ('get', 'put', 'delete'):
            try:
                getattr(self.webserver, request_method)(path)
                self.fail('%r are not allowed for this path %r' % (
                    request_method, path))
            except AppError as e:
                if '404' not in e.args[0]:
                    raise

    def test_REst_delete(self):
        path = '/test/rest/delete'
        self.init_registry(self.add_in_registry_rest, path=path,
                           request_method='DELETE')
        response = self.webserver.delete(path)
        self.assertEqual(response.status, '200 OK')
        for request_method in ('get', 'put', 'post'):
            try:
                getattr(self.webserver, request_method)(path)
                self.fail('%r are not allowed for this path %r' % (
                    request_method, path))
            except AppError as e:
                if '404' not in e.args[0]:
                    raise



# Import anyblok_pyramid to declare the Core and Entry types
import anyblok_pyramid  # noqa
from anyblok.tests.testcase import TestCase
from anyblok.registry import RegistryManager
from anyblok.environment import EnvironmentManager
from anyblok import Declarations
register = Declarations.register
unregister = Declarations.unregister
PyramidHTTP = Declarations.PyramidHTTP


class OneController:
    pass


class TestPyramidHTTP(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TestPyramidHTTP, cls).setUpClass()
        RegistryManager.init_blok('testPyramidHTTP')
        EnvironmentManager.set('current_blok', 'testPyramidHTTP')
        cls.views = PyramidHTTP.views.copy()
        cls.routes = [] + PyramidHTTP.routes

    @classmethod
    def tearDownClass(cls):
        super(TestPyramidHTTP, cls).tearDownClass()
        EnvironmentManager.set('current_blok', None)
        del RegistryManager.loaded_bloks['testPyramidHTTP']
        PyramidHTTP.views = cls.views
        PyramidHTTP.routes = cls.routes

    def setUp(self):
        super(TestPyramidHTTP, self).setUp()
        blokname = 'testPyramidHTTP'
        RegistryManager.loaded_bloks[blokname]['PyramidHTTP'] = {
            'registry_names': []}

    def assertInPyramidHTTP(self, *args):
        blokname = 'testPyramidHTTP'
        blok = RegistryManager.loaded_bloks[blokname]
        bases = blok['PyramidHTTP']['PyramidHTTP.MyController']['bases']
        self.assertEqual(len(bases), len(args))
        for cls_ in args:
            has = cls_ in bases
            self.assertEqual(has, True)

    def assertInRemoved(self, cls):
        core = RegistryManager.loaded_bloks['testPyramidHTTP']['removed']
        if cls in core:
            return True

        self.fail('Not in removed')

    def test_add_interface(self):
        register(PyramidHTTP, cls_=OneController, name_='MyController')
        self.assertEqual('PyramidHTTP',
                         PyramidHTTP.MyController.__declaration_type__)
        self.assertInPyramidHTTP(OneController)
        dir(Declarations.PyramidHTTP.MyController)

    def test_add_interface_with_decorator(self):

        @register(PyramidHTTP)
        class MyController:
            pass

        self.assertEqual('PyramidHTTP',
                         PyramidHTTP.MyController.__declaration_type__)
        self.assertInPyramidHTTP(MyController)

    def test_add_two_interface(self):

        register(PyramidHTTP, cls_=OneController, name_="MyController")

        @register(PyramidHTTP)
        class MyController:
            pass

        self.assertInPyramidHTTP(OneController, MyController)

    def test_remove_interface_with_1_cls_in_registry(self):

        register(PyramidHTTP, cls_=OneController, name_="MyController")
        self.assertInPyramidHTTP(OneController)
        unregister(PyramidHTTP.MyController, OneController)
        self.assertInPyramidHTTP(OneController)
        self.assertInRemoved(OneController)

    def test_remove_interface_with_2_cls_in_registry(self):

        register(PyramidHTTP, cls_=OneController, name_="MyController")

        @register(PyramidHTTP)
        class MyController:
            pass

        self.assertInPyramidHTTP(OneController, MyController)
        unregister(PyramidHTTP.MyController, OneController)
        self.assertInPyramidHTTP(MyController, OneController)
        self.assertInRemoved(OneController)



from anyblok_pyramid.tests.testcase import PyramidDBTestCase
from anyblok import Declarations
register = Declarations.register
PyramidJsonRPC = Declarations.PyramidJsonRPC
PyramidMixin = Declarations.PyramidMixin
Core = Declarations.Core


class TestDeclarationPyramidJsonRPC(PyramidDBTestCase):

    def check_controller(self):
        res = self.jsonrpc('/test', 'methodA', params={'a': 2, 'b': 3})
        self.assertEqual(res['result'], {'a': 4, 'b': 6})
        res = self.jsonrpc('/test', 'methodB', params={'a': 2, 'b': 3})
        self.assertEqual(res['result'], {'a': 6, 'b': 9})

    def test_jsonrpc(self):
        def add_jsonrpc_contoller():

            @register(PyramidJsonRPC)
            class Test:

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_jsonrpc_without_routes(self):
        def add_jsonrpc_contoller():

            @register(PyramidJsonRPC)
            class Test:

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

        with self.assertRaises(Declarations.Exception.PyramidException):
            self.init_registry(add_jsonrpc_contoller)

    def test_simple_subclass_controller(self):
        def add_jsonrpc_contoller():

            @register(PyramidJsonRPC)
            class Test:

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)  # noqa
            class Test:

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_simple_subclass_base_jsonrpc(self):
        def add_jsonrpc_contoller():

            @register(Core)
            class PyramidBaseJsonRPC:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test:

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_simple_subclass_base_rpc(self):
        def add_jsonrpc_contoller():

            @register(Core)
            class PyramidBaseRPC:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test:

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_mixin_one_controller(self):
        def add_jsonrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test(PyramidMixin.PyMixin):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_mixin_two_controller(self):
        def add_jsonrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test(PyramidMixin.PyMixin):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test2(PyramidMixin.PyMixin):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test2, self).methodA(**kwargs)

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')
            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test2, '/test2')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()
        res = self.jsonrpc('/test2', 'methodA', params={'a': 2, 'b': 3})
        self.assertEqual(res['result'], {'a': 4, 'b': 6})

    def test_mixin_one_controller_with_subclass(self):
        def add_jsonrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test(PyramidMixin.PyMixin):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

            def inherit_Test():
                # put in the function else python take only on the first
                # Test class, because the fingerprint are the same
                @register(PyramidJsonRPC)
                class Test:

                    @PyramidJsonRPC.rpc_method(method='methodB')
                    def method_B(self, **kwargs):
                        return {x: y * 3 for x, y in kwargs.items()}

            inherit_Test()

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_mixin_one_controller_by_subclass(self):
        def add_jsonrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test:

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_mixin_one_controller_by_subclass_and_with(self):
        def add_jsonrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test:

                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

            def inherit_Test():
                # put in the function else python take only on the first
                # Test class, because the fingerprint are the same
                @register(PyramidJsonRPC)
                class Test:

                    @PyramidJsonRPC.rpc_method(method='methodB')
                    def method_B(self, **kwargs):
                        return super(Test, self).method_B(**kwargs)

            inherit_Test()

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_mixin_one_controller_with_subclass_and_subclass_mixin(self):
        def add_jsonrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return None

            @register(PyramidJsonRPC)
            class Test:

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

            @register(PyramidMixin)  # noqa
            class PyMixin:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_inherit_by_another_controller(self):
        def add_jsonrpc_contoller():

            @register(PyramidJsonRPC)
            class Test2:

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test(PyramidJsonRPC.Test2):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return super(Test, self).method_B(**kwargs)

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')
            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test2, '/test2')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_inherit_by_another_controller_and_subclass_maincontroller(self):
        def add_jsonrpc_contoller():

            @register(PyramidJsonRPC)
            class Test2:

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            @register(PyramidJsonRPC)
            class Test(PyramidJsonRPC.Test2):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return super(Test, self).method_B(**kwargs)

            @register(PyramidJsonRPC)  # noqa
            class Test2:

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')
            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test2, '/test2')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()

    def test_inherit_core_and_mixin(self):
        def add_jsonrpc_contoller():

            @register(Core)
            class PyramidBaseRPC:

                def methodA(self, **kwargs):
                    return {x: y * 2 for x, y in kwargs.items()}

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, **kwargs):
                    return super(PyMixin, self).methodA(**kwargs)

            @register(PyramidJsonRPC)
            class Test(PyramidMixin.PyMixin):

                @PyramidJsonRPC.rpc_method()
                def methodA(self, **kwargs):
                    return super(Test, self).methodA(**kwargs)

                @PyramidJsonRPC.rpc_method(method='methodB')
                def method_B(self, **kwargs):
                    return {x: y * 3 for x, y in kwargs.items()}

            Declarations.PyramidJsonRPC.add_route(
                Declarations.PyramidJsonRPC.Test, '/test')

        self.init_registry(add_jsonrpc_contoller)
        self.check_controller()



from anyblok_pyramid.tests.testcase import PyramidDBTestCase
from anyblok import Declarations
register = Declarations.register
PyramidJsonRPC = Declarations.PyramidJsonRPC
PyramidMixin = Declarations.PyramidMixin
Core = Declarations.Core


class TestSimpleCache(PyramidDBTestCase):

    def call_json(self, value):
        res = self.jsonrpc('/test', 'get_method_cached')
        self.assertEqual(res['result'], value)

    def check_method_cached(self, registry, registry_name, value=1):
        self.call_json(value)
        self.call_json(value)
        registry.System.Cache.invalidate(registry_name, 'method_cached')
        self.call_json(value * 2)

    def add_controller(self):
        @register(PyramidJsonRPC)
        class Test:

            @PyramidJsonRPC.rpc_method()
            def get_method_cached(self):
                return self.method_cached()

        PyramidJsonRPC.add_route(PyramidJsonRPC.Test, '/test')

    def test_model(self):
        def add_model_with_method_cached():

            @register(PyramidJsonRPC)
            class Test:

                x = 0

                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

            self.add_controller()

        registry = self.init_registry(add_model_with_method_cached)
        self.check_method_cached(registry, 'PyramidJsonRPC.Test')

    def test_core(self):
        def add_model_with_method_cached_by_core():
            @register(Core)
            class PyramidBaseRPC:

                x = 0

                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

            self.add_controller()

        registry = self.init_registry(add_model_with_method_cached_by_core)
        self.check_method_cached(registry, 'PyramidJsonRPC.Test')

    def test_mixin(self):
        def add_model_with_method_cached_by_mixin():
            @register(PyramidMixin)
            class MTest:

                x = 0

                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

            @register(PyramidJsonRPC)
            class Test(PyramidMixin.MTest):
                pass

            self.add_controller()

        registry = self.init_registry(add_model_with_method_cached_by_mixin)
        self.check_method_cached(registry, 'PyramidJsonRPC.Test')

    def add_model_with_method_cached_with_mixin_and_or_core(self,
                                                            withmodel=False,
                                                            withmixin=False,
                                                            withcore=False):

        @register(Core)
        class PyramidBaseRPC:

            x = 0

            if withcore:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

            else:
                @classmethod
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

        @register(PyramidMixin)
        class MTest:

            y = 0

            if withmixin:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(MTest, cls).method_cached()

            else:
                @classmethod
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(MTest, cls).method_cached()

        @register(PyramidJsonRPC)
        class Test(PyramidMixin.MTest):

            z = 0

            if withmodel:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.z += 3
                    return cls.z + super(Test, cls).method_cached()
            else:
                @classmethod
                def method_cached(cls):
                    cls.z += 3
                    return cls.z + super(Test, cls).method_cached()

        self.add_controller()

    def test_model_mixin_core_not_cache(self):
        self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core)
        self.call_json(6)
        self.call_json(12)

    def test_model_mixin_core_only_core(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core,
            withcore=True)
        self.call_json(6)
        self.call_json(11)
        registry.System.Cache.invalidate('PyramidJsonRPC.Test',
                                         'method_cached')
        self.call_json(17)

    def test_model_mixin_core_only_mixin(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core,
            withmixin=True)
        self.call_json(6)
        self.call_json(9)
        registry.System.Cache.invalidate('PyramidJsonRPC.Test',
                                         'method_cached')
        self.call_json(15)

    def test_model_mixin_core_only_model(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core,
            withmodel=True)
        self.call_json(6)
        self.call_json(6)
        registry.System.Cache.invalidate('PyramidJsonRPC.Test',
                                         'method_cached')
        self.call_json(12)

    def test_model_mixin_core_only_core_and_mixin(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core,
            withmixin=True, withcore=True)
        self.call_json(6)
        self.call_json(9)
        registry.System.Cache.invalidate('PyramidJsonRPC.Test',
                                         'method_cached')
        self.call_json(15)


class TestInheritedCache(PyramidDBTestCase):

    def call_json(self, value):
        res = self.jsonrpc('/test', 'get_method_cached')
        self.assertEqual(res['result'], value)

    def check_method_cached(self, registry):
        self.call_json(3)
        self.call_json(5)
        registry.System.Cache.invalidate('PyramidJsonRPC.Test',
                                         'method_cached')
        self.call_json(8)

    def check_inherited_method_cached(self, registry):
        self.call_json(3)
        self.call_json(3)
        registry.System.Cache.invalidate('PyramidJsonRPC.Test',
                                         'method_cached')
        self.call_json(6)

    def add_controller(self):
        @register(PyramidJsonRPC)
        class Test:

            @PyramidJsonRPC.rpc_method()
            def get_method_cached(self):
                res = self.method_cached()
                return res

        PyramidJsonRPC.add_route(PyramidJsonRPC.Test, '/test')

    def add_model_with_method_cached(self, inheritcache=False):

        @register(PyramidJsonRPC)
        class Test:

            x = 0

            @Declarations.classmethod_cache()
            def method_cached(cls):
                cls.x += 1
                return cls.x

        @register(PyramidJsonRPC)  # noqa
        class Test:

            y = 0

            if inheritcache:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(Test, cls).method_cached()
            else:
                @classmethod
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(Test, cls).method_cached()

        self.add_controller()

    def add_model_with_method_cached_by_core(self, inheritcache=False):

        @register(Core)
        class PyramidBaseRPC:

            x = 0

            @Declarations.classmethod_cache()
            def method_cached(cls):
                cls.x += 1
                return cls.x

        @register(Core)  # noqa
        class PyramidBaseRPC:

            y = 0

            if inheritcache:
                @Declarations.classmethod_cache()
                def method_cached(self):
                    self.y += 2
                    return self.y + super(PyramidBaseRPC, self).method_cached()
            else:
                @classmethod
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(PyramidBaseRPC, cls).method_cached()

        self.add_controller()

    def add_model_with_method_cached_by_mixin(self, inheritcache=False):

        @register(PyramidMixin)
        class MTest:

            x = 0

            @Declarations.classmethod_cache()
            def method_cached(cls):
                cls.x += 1
                return cls.x

        @register(PyramidMixin)  # noqa
        class MTest:

            y = 0

            if inheritcache:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(MTest, cls).method_cached()
            else:
                @classmethod
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(MTest, cls).method_cached()

        @register(PyramidJsonRPC)
        class Test(PyramidMixin.MTest):
            pass

        self.add_controller()

    def test_model(self):
        registry = self.init_registry(self.add_model_with_method_cached)
        self.check_method_cached(registry)

    def test_model2(self):
        registry = self.init_registry(self.add_model_with_method_cached,
                                      inheritcache=True)
        self.check_inherited_method_cached(registry)

    def test_core(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_by_core)
        self.check_method_cached(registry)

    def test_core2(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_by_core, inheritcache=True)
        self.check_inherited_method_cached(registry)

    def test_mixin(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_by_mixin)
        self.check_method_cached(registry)

    def test_mixin2(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_by_mixin, inheritcache=True)
        self.check_inherited_method_cached(registry)



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from wsgiref.simple_server import make_server
from anyblok.scripts import format_argsparse, createdb, run_exit
from anyblok.blok import BlokManager
from anyblok.registry import RegistryManager
from anyblok._argsparse import ArgsParseManager
from .config import make_config


def anyblok_wsgi(description, version, argsparse_groups, parts_to_load,
                 Configurator=make_config):
    """

    :param description: description of argsparse
    :param version: version of script for argparse
    :param argsparse_groups: list argsparse groupe to load
    :param parts_to_load: group of blok to load
    :param Configurator: callable which return a config instance
    """
    format_argsparse(argsparse_groups, 'wsgi', 'pyramid-debug', 'beaker')
    BlokManager.load(*parts_to_load)
    ArgsParseManager.load(description="%s (%s)" % (description, version),
                          argsparse_groups=argsparse_groups,
                          parts_to_load=parts_to_load)
    RegistryManager.add_needed_bloks('pyramid')
    config = Configurator()
    wsgi_host = ArgsParseManager.get('wsgi_host')
    wsgi_port = int(ArgsParseManager.get('wsgi_port'))

    app = config.make_wsgi_app()
    server = make_server(wsgi_host, wsgi_port, app)

    dbnames = ArgsParseManager.get('dbnames', '').split(',')
    dbname = ArgsParseManager.get('dbname')
    if dbname not in dbnames:
        dbnames.append(dbname)

    # preload all db names
    for dbname in [x for x in dbnames if x != '']:
        RegistryManager.get(dbname)

    server.serve_forever()


def wsgi():
    anyblok_wsgi('Web server for AnyBlok', '0.0.1',
                 ['config', 'database'], ['AnyBlok'])


def anyblok_createdb():
    from anyblok_pyramid.release import version
    description = "Anyblok / Pyramid - %s create db" % version
    createdb(description, ['config', 'database', 'unittest'], ['AnyBlok'])


def anyblok_nose():
    from anyblok_pyramid.release import version
    run_exit("Nose test for AnyBlok / Pyramid", version,
             ['wsgi', 'config', 'database'], ['AnyBlok'])



# Import anyblok_pyramid to declare the Core and Entry types
import anyblok_pyramid  # noqa
from anyblok.tests.testcase import TestCase
from anyblok.registry import RegistryManager
from anyblok.environment import EnvironmentManager
from anyblok import Declarations
register = Declarations.register
unregister = Declarations.unregister
PyramidJsonRPC = Declarations.PyramidJsonRPC


class OneController:
    pass


class TestPyramidJsonRPC(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TestPyramidJsonRPC, cls).setUpClass()
        RegistryManager.init_blok('testPyramidJsonRPC')
        EnvironmentManager.set('current_blok', 'testPyramidJsonRPC')
        cls.methods = PyramidJsonRPC.methods.copy()
        cls.routes = [] + PyramidJsonRPC.routes

    @classmethod
    def tearDownClass(cls):
        super(TestPyramidJsonRPC, cls).tearDownClass()
        EnvironmentManager.set('current_blok', None)
        del RegistryManager.loaded_bloks['testPyramidJsonRPC']
        PyramidJsonRPC.methods = cls.methods
        PyramidJsonRPC.routes = cls.routes

    def setUp(self):
        super(TestPyramidJsonRPC, self).setUp()
        blokname = 'testPyramidJsonRPC'
        RegistryManager.loaded_bloks[blokname]['PyramidJsonRPC'] = {
            'registry_names': []}

    def assertInPyramidJsonRPC(self, *args):
        blokname = 'testPyramidJsonRPC'
        blok = RegistryManager.loaded_bloks[blokname]
        bases = blok['PyramidJsonRPC']['PyramidJsonRPC.MyController']['bases']
        self.assertEqual(len(bases), len(args))
        for cls_ in args:
            has = cls_ in bases
            self.assertEqual(has, True)

    def assertInRemoved(self, cls):
        core = RegistryManager.loaded_bloks['testPyramidJsonRPC']['removed']
        if cls in core:
            return True

        self.fail('Not in removed')

    def test_add_interface(self):
        register(PyramidJsonRPC, cls_=OneController, name_='MyController')
        self.assertEqual('PyramidJsonRPC',
                         PyramidJsonRPC.MyController.__declaration_type__)
        self.assertInPyramidJsonRPC(OneController)
        dir(Declarations.PyramidJsonRPC.MyController)

    def test_add_interface_with_decorator(self):

        @register(PyramidJsonRPC)
        class MyController:
            pass

        self.assertEqual('PyramidJsonRPC',
                         PyramidJsonRPC.MyController.__declaration_type__)
        self.assertInPyramidJsonRPC(MyController)

    def test_add_two_interface(self):

        register(PyramidJsonRPC, cls_=OneController, name_="MyController")

        @register(PyramidJsonRPC)
        class MyController:
            pass

        self.assertInPyramidJsonRPC(OneController, MyController)

    def test_remove_interface_with_1_cls_in_registry(self):

        register(PyramidJsonRPC, cls_=OneController, name_="MyController")
        self.assertInPyramidJsonRPC(OneController)
        unregister(PyramidJsonRPC.MyController, OneController)
        self.assertInPyramidJsonRPC(OneController)
        self.assertInRemoved(OneController)

    def test_remove_interface_with_2_cls_in_registry(self):

        register(PyramidJsonRPC, cls_=OneController, name_="MyController")

        @register(PyramidJsonRPC)
        class MyController:
            pass

        self.assertInPyramidJsonRPC(OneController, MyController)
        unregister(PyramidJsonRPC.MyController, OneController)
        self.assertInPyramidJsonRPC(MyController, OneController)
        self.assertInRemoved(OneController)



# Import anyblok_pyramid to declare the Core and Entry types
import anyblok_pyramid  # noqa
from anyblok.tests.testcase import TestCase
from anyblok.registry import RegistryManager
from anyblok.environment import EnvironmentManager
from anyblok import Declarations
register = Declarations.register
unregister = Declarations.unregister
PyramidXmlRPC = Declarations.PyramidXmlRPC


class OneController:
    pass


class TestPyramidXmlRPC(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TestPyramidXmlRPC, cls).setUpClass()
        RegistryManager.init_blok('testPyramidXmlRPC')
        EnvironmentManager.set('current_blok', 'testPyramidXmlRPC')
        cls.methods = PyramidXmlRPC.methods.copy()
        cls.routes = [] + PyramidXmlRPC.routes

    @classmethod
    def tearDownClass(cls):
        super(TestPyramidXmlRPC, cls).tearDownClass()
        EnvironmentManager.set('current_blok', None)
        del RegistryManager.loaded_bloks['testPyramidXmlRPC']
        PyramidXmlRPC.methods = cls.methods
        PyramidXmlRPC.routes = cls.routes

    def setUp(self):
        super(TestPyramidXmlRPC, self).setUp()
        blokname = 'testPyramidXmlRPC'
        RegistryManager.loaded_bloks[blokname]['PyramidXmlRPC'] = {
            'registry_names': []}

    def assertInPyramidXmlRPC(self, *args):
        blokname = 'testPyramidXmlRPC'
        blok = RegistryManager.loaded_bloks[blokname]
        bases = blok['PyramidXmlRPC']['PyramidXmlRPC.MyController']['bases']
        self.assertEqual(len(bases), len(args))
        for cls_ in args:
            has = cls_ in bases
            self.assertEqual(has, True)

    def assertInRemoved(self, cls):
        core = RegistryManager.loaded_bloks['testPyramidXmlRPC']['removed']
        if cls in core:
            return True

        self.fail('Not in removed')

    def test_add_interface(self):
        register(PyramidXmlRPC, cls_=OneController, name_='MyController')
        self.assertEqual('PyramidXmlRPC',
                         PyramidXmlRPC.MyController.__declaration_type__)
        self.assertInPyramidXmlRPC(OneController)
        dir(Declarations.PyramidXmlRPC.MyController)

    def test_add_interface_with_decorator(self):

        @register(PyramidXmlRPC)
        class MyController:
            pass

        self.assertEqual('PyramidXmlRPC',
                         PyramidXmlRPC.MyController.__declaration_type__)
        self.assertInPyramidXmlRPC(MyController)

    def test_add_two_interface(self):

        register(PyramidXmlRPC, cls_=OneController, name_="MyController")

        @register(PyramidXmlRPC)
        class MyController:
            pass

        self.assertInPyramidXmlRPC(OneController, MyController)

    def test_remove_interface_with_1_cls_in_registry(self):

        register(PyramidXmlRPC, cls_=OneController, name_="MyController")
        self.assertInPyramidXmlRPC(OneController)
        unregister(PyramidXmlRPC.MyController, OneController)
        self.assertInPyramidXmlRPC(OneController)
        self.assertInRemoved(OneController)

    def test_remove_interface_with_2_cls_in_registry(self):

        register(PyramidXmlRPC, cls_=OneController, name_="MyController")

        @register(PyramidXmlRPC)
        class MyController:
            pass

        self.assertInPyramidXmlRPC(OneController, MyController)
        unregister(PyramidXmlRPC.MyController, OneController)
        self.assertInPyramidXmlRPC(MyController, OneController)
        self.assertInRemoved(OneController)



from anyblok.blok import Blok
from anyblok_pyramid.release import version


class Pyramid(Blok):
    """
    Server tools to use the Pyramid views and routes declarations with
    the AnyBlok framework
    """
    version = version

    required = [
        'anyblok-core',
    ]

    @classmethod
    def import_declaration_module(cls):
        from . import base  # noqa

    @classmethod
    def reload_declaration_module(cls, reload):
        from . import base
        reload(base)



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from anyblok._argsparse import ArgsParseManager
import os


@ArgsParseManager.add('wsgi', label="WSGI")
def define_wsgi_option(group, configuration):
    group.add_argument('--databases', dest='dbnames', default='',
                       help='List of the database allow to be load')
    group.add_argument('--wsgi-host', dest='wsgi_host', default='')
    group.add_argument('--wsgi-port', dest='wsgi_port', default='')
    configuration.update({
        'wsgi_host': os.environ.get('ANYBLOK_PYRAMID_WSGI_HOST', 'localhost'),
        'wsgi_port': os.environ.get('ANYBLOK_PYRAMID_WSGI_PORT', '5000'),
    })


@ArgsParseManager.add('pyramid-debug', label="Pyramid")
def define_wsgi_debug_option(group, configuration):
    group.add_argument('--pyramid-reload-all', dest='pyramid.reload_all',
                       help="Turns on all reload* settings.",
                       action='store_true')
    group.add_argument('--pyramid-reload-templates',
                       dest='pyramid.reload_templates',
                       help="templates are "
                            "automatically reloaded whenever they are "
                            "modified without restarting the application, so "
                            "you can see changes to templates take effect "
                            "immediately during development. This flag is "
                            "meaningful to Chameleon and Mako templates, as "
                            "well as most third-party template rendering "
                            "extensions.",
                       action='store_true')
    group.add_argument('--pyramid-reload-assets',
                       dest='pyramid.reload_assets',
                       help="Don't cache any asset file data",
                       action='store_true')
    group.add_argument('--pyramid-debug-all',
                       dest='pyramid.debug_all',
                       help="Turns on all debug* settings.",
                       action='store_true')
    group.add_argument('--pyramid-debug-notfound',
                       dest='pyramid.debug_notfound',
                       help="Print view-related NotFound debug messages "
                            "to stderr",
                       action='store_true')
    group.add_argument('--pyramid-debug-routematch',
                       dest='pyramid.debug_routematch',
                       help="Print debugging messages related to url dispatch "
                            "route matching",
                       action='store_true')
    group.add_argument('--pyramid-prevent-http-cache',
                       dest='pyramid.prevent_http_cache',
                       help="Prevent the http_cache view configuration "
                            "argument from having any effect globally in this "
                            "process when this value is true. No http "
                            "caching-related response headers will be set by "
                            "the Pyramid http_cache view configuration "
                            "feature",
                       action='store_true')
    group.add_argument('--pyramid-default-locale-name',
                       dest='pyramid.default_locale_name',
                       help="The value supplied here is used as the default "
                            "locale name when a locale negotiator is not "
                            "registered.",
                       default='')
    configuration.update({
        'pyramid.reload_all': False,
        'pyramid.reload_templates': False,
        'pyramid.reload_assets': False,
        'pyramid.debug_all': False,
        'pyramid.debug_notfound': False,
        'pyramid.debug_routematch': False,
        'pyramid.prevent_http_cache': False,
        'pyramid.default_locale_name': '',
    })


@ArgsParseManager.add('beaker', label="Beaker session")
def define_beaker_option(group, configuration):
    group.add_argument('--beaker-data-dir',
                       dest='beaker.session.data_dir',
                       help="Used with any back-end that stores its data in "
                            "physical files, such as the dbm or file-based "
                            "back-ends. This path should be an absolute path "
                            "to the directory that stores the files.")
    group.add_argument('--beaker-lock-dir',
                       dest='beaker.session.lock_dir',
                       help="Used with every back-end, to coordinate locking. "
                            "With caching, this lock file is used to ensure "
                            "that multiple processes/threads aren’t "
                            "attempting to re-create the same value at the "
                            "same time (The Dog-Pile Effect)")
    group.add_argument('--beaker-memcache-module',
                       dest='beaker.session.memcache_module',
                       help="One of the names memcache, cmemcache, pylibmc, "
                            "or auto. Default is auto. Specifies which "
                            "memcached client library should be imported when "
                            "using the ext:memcached backend. If left at its "
                            "default of auto, pylibmc is favored first, then "
                            "cmemcache, then memcache.")
    group.add_argument('--beaker-type',
                       dest='beaker.session.type',
                       help="The name of the back-end to use for storing the "
                            "sessions or cache objects.\nAvailable back-ends "
                            "supplied with Beaker: file, dbm, memory, "
                            "ext:memcached, ext:database, ext:google\nFor "
                            "sessions, the additional type of cookie is "
                            "available which will store all the session data "
                            "in the cookie itself. As such, size limitations "
                            "apply (4096 bytes).\nSome of these back-ends "
                            "require the url option as listed below.")
    group.add_argument('--beaker-url',
                       dest='beaker.session.url',
                       help="URL is specific to use of either ext:memcached "
                            "or ext:database. When using one of those types, "
                            "this option is required. When used with "
                            "ext:memcached, this should be either a single, "
                            "or semi-colon separated list of memcached "
                            "servers When used with ext:database, this should "
                            "be a valid SQLAlchemy database string.")
    group.add_argument('--beaker-cookie-expires',
                       dest='beaker.session.cookie_expires',
                       help="Determines when the cookie used to track the "
                            "client-side of the session will expire. When set "
                            "to a boolean value, it will either expire at the "
                            "end of the browsers session, or never expire. "
                            "Setting to a datetime forces a hard ending time "
                            "for the session (generally used for setting a "
                            "session to a far off date). Setting to an "
                            "integer will result in the cookie being set to "
                            "expire in that many seconds. I.e. a value of 300 "
                            "will result in the cookie being set to expire in "
                            "300 seconds. Defaults to never expiring.")
    group.add_argument('--beaker-cookie-domain',
                       dest='beaker.session.cookie_domain',
                       help="What domain the cookie should be set to. When "
                            "using sub-domains, this should be set to the "
                            "main domain the cookie should be valid for. For "
                            "example, if a cookie should be valid under "
                            "www.nowhere.com and files.nowhere.com then it "
                            "should be set to .nowhere.com. Defaults to the "
                            "current domain in its entirety.")
    group.add_argument('--beaker-key',
                       dest='beaker.session.key',
                       help="Name of the cookie key used to save the session "
                            "under.")
    group.add_argument('--beaker-secret',
                       dest='beaker.session.secret',
                       help="Used with the HMAC to ensure session integrity. "
                            "This value should ideally be a randomly "
                            "generated string. When using in a cluster "
                            "environment, the secret must be the same on "
                            "every machine.")
    group.add_argument('--beaker-secure',
                       dest='beaker.session.secure',
                       help="Whether or not the session cookie should be "
                            "marked as secure. When marked as secure, "
                            "browsers are instructed to not send the cookie "
                            "over anything other than an SSL connection.")
    group.add_argument('--beaker-timeout',
                       dest='beaker.session.timeout',
                       help="Seconds until the session is considered invalid, "
                            "after which it will be ignored and invalidated. "
                            "This number is based on the time since the "
                            "session was last accessed, not from when the "
                            "session was created. Defaults to never expiring.")
    group.add_argument('--beaker-encrypt-key',
                       dest='beaker.session.encrypt_key',
                       help="Encryption key to use for the AES cipher. This "
                            "should be a fairly long randomly generated "
                            "string.")
    group.add_argument('--beaker-validate-key',
                       dest='beaker.session.validate_key',
                       help="Validation key used to sign the AES encrypted "
                            "data.")



# This file is a part of the AnyBlok / Pyramid project
#
#    Copyright (C) 2015 Jean-Sebastien SUZANNE <jssuzanne@anybox.fr>
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file,You can
# obtain one at http://mozilla.org/MPL/2.0/.
from setuptools import setup, find_packages
import os
version = '0.3.1'

requires = [
    'anyblok',
    'pyramid',
    'pyramid_beaker',
    'pyramid_rpc',
]

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as readme:
    README = readme.read()

with open(os.path.join(here, 'doc', 'FRONT.rst')) as front:
    FRONT = front.read()

with open(os.path.join(here, 'doc', 'CHANGES.rst')) as change:
    CHANGE = change.read()

console_scripts = [
    'anyblok_pyramid=anyblok_pyramid.scripts:wsgi',
    'anyblok_pyramid_createdb=anyblok_pyramid.scripts:anyblok_createdb',
    'anyblok_pyramid_nose=anyblok_pyramid.scripts:anyblok_nose',
]

setup(
    name="anyblok_pyramid",
    version=version,
    author="ean-Sébastien Suzanne",
    author_email="jssuzanne@anybox.fr",
    description="Web Server Pyramid for AnyBlok",
    license="MPL2",
    long_description=README + '\n' + FRONT + '\n' + CHANGE,
    url="https://bitbucket.org/jsuzanne/anyblok_pyramid",
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=requires,
    tests_require=requires + ['nose', 'WebTest'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Framework :: Pyramid',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
        'Topic :: Internet :: WWW/HTTP :: Session',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Server',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
    ],
    entry_points={
        'console_scripts': console_scripts,
        'AnyBlok': [
            'pyramid=anyblok_pyramid.bloks.pyramid:Pyramid',
        ],
    },
    extras_require={},
)



from anyblok_pyramid.tests.testcase import PyramidDBTestCase
from anyblok import Declarations
from pyramid.response import Response
register = Declarations.register
PyramidHTTP = Declarations.PyramidHTTP
PyramidMixin = Declarations.PyramidMixin
Core = Declarations.Core


class TestSimpleCache(PyramidDBTestCase):

    def call_http(self, value):
        res = self.http('/test', method='get')
        self.assertEqual(eval(res.body), value)

    def check_method_cached(self, registry, registry_name, value=1):
        self.call_http(value)
        self.call_http(value)
        registry.System.Cache.invalidate(registry_name, 'method_cached')
        self.call_http(value * 2)

    def add_controller(self):
        @register(PyramidHTTP)
        class Test:

            @PyramidHTTP.view()
            def get_method_cached(self):
                return Response(str(self.method_cached()))

        PyramidHTTP.add_route('get_method_cached', '/test')

    def test_model(self):
        def add_model_with_method_cached():

            @register(PyramidHTTP)
            class Test:

                x = 0

                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

            self.add_controller()

        registry = self.init_registry(add_model_with_method_cached)
        self.check_method_cached(registry, 'PyramidHTTP.Test')

    def test_core(self):
        def add_model_with_method_cached_by_core():
            @register(Core)
            class PyramidBaseHTTP:

                x = 0

                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

            self.add_controller()

        registry = self.init_registry(add_model_with_method_cached_by_core)
        self.check_method_cached(registry, 'PyramidHTTP.Test')

    def test_mixin(self):
        def add_model_with_method_cached_by_mixin():
            @register(PyramidMixin)
            class MTest:

                x = 0

                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

            @register(PyramidHTTP)
            class Test(PyramidMixin.MTest):
                pass

            self.add_controller()

        registry = self.init_registry(add_model_with_method_cached_by_mixin)
        self.check_method_cached(registry, 'PyramidHTTP.Test')

    def add_model_with_method_cached_with_mixin_and_or_core(self,
                                                            withmodel=False,
                                                            withmixin=False,
                                                            withcore=False):

        @register(Core)
        class PyramidBaseHTTP:

            x = 0

            if withcore:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

            else:
                @classmethod
                def method_cached(cls):
                    cls.x += 1
                    return cls.x

        @register(PyramidMixin)
        class MTest:

            y = 0

            if withmixin:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(MTest, cls).method_cached()

            else:
                @classmethod
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(MTest, cls).method_cached()

        @register(PyramidHTTP)
        class Test(PyramidMixin.MTest):

            z = 0

            if withmodel:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.z += 3
                    return cls.z + super(Test, cls).method_cached()
            else:
                @classmethod
                def method_cached(cls):
                    cls.z += 3
                    return cls.z + super(Test, cls).method_cached()

        self.add_controller()

    def test_model_mixin_core_not_cache(self):
        self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core)
        self.call_http(6)
        self.call_http(12)

    def test_model_mixin_core_only_core(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core,
            withcore=True)
        self.call_http(6)
        self.call_http(11)
        registry.System.Cache.invalidate('PyramidHTTP.Test',
                                         'method_cached')
        self.call_http(17)

    def test_model_mixin_core_only_mixin(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core,
            withmixin=True)
        self.call_http(6)
        self.call_http(9)
        registry.System.Cache.invalidate('PyramidHTTP.Test',
                                         'method_cached')
        self.call_http(15)

    def test_model_mixin_core_only_model(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core,
            withmodel=True)
        self.call_http(6)
        self.call_http(6)
        registry.System.Cache.invalidate('PyramidHTTP.Test',
                                         'method_cached')
        self.call_http(12)

    def test_model_mixin_core_only_core_and_mixin(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_with_mixin_and_or_core,
            withmixin=True, withcore=True)
        self.call_http(6)
        self.call_http(9)
        registry.System.Cache.invalidate('PyramidHTTP.Test',
                                         'method_cached')
        self.call_http(15)


class TestInheritedCache(PyramidDBTestCase):

    def call_http(self, value):
        res = self.http('/test', method='get')
        self.assertEqual(eval(res.body), value)

    def check_method_cached(self, registry):
        self.call_http(3)
        self.call_http(5)
        registry.System.Cache.invalidate('PyramidHTTP.Test',
                                         'method_cached')
        self.call_http(8)

    def check_inherited_method_cached(self, registry):
        self.call_http(3)
        self.call_http(3)
        registry.System.Cache.invalidate('PyramidHTTP.Test',
                                         'method_cached')
        self.call_http(6)

    def add_controller(self):
        @register(PyramidHTTP)
        class Test:

            @PyramidHTTP.view()
            def get_method_cached(self):
                return Response(str(self.method_cached()))

        PyramidHTTP.add_route('get_method_cached', '/test')

    def add_model_with_method_cached(self, inheritcache=False):

        @register(PyramidHTTP)
        class Test:

            x = 0

            @Declarations.classmethod_cache()
            def method_cached(cls):
                cls.x += 1
                return cls.x

        @register(PyramidHTTP)  # noqa
        class Test:

            y = 0

            if inheritcache:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(Test, cls).method_cached()
            else:
                @classmethod
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(Test, cls).method_cached()

        self.add_controller()

    def add_model_with_method_cached_by_core(self, inheritcache=False):

        @register(Core)
        class PyramidBaseHTTP:

            x = 0

            @Declarations.classmethod_cache()
            def method_cached(cls):
                cls.x += 1
                return cls.x

        @register(Core)  # noqa
        class PyramidBaseHTTP:

            y = 0

            if inheritcache:
                @Declarations.classmethod_cache()
                def method_cached(self):
                    self.y += 2
                    return self.y + super(PyramidBaseHTTP,
                                          self).method_cached()
            else:
                @classmethod
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(PyramidBaseHTTP, cls).method_cached()

        self.add_controller()

    def add_model_with_method_cached_by_mixin(self, inheritcache=False):

        @register(PyramidMixin)
        class MTest:

            x = 0

            @Declarations.classmethod_cache()
            def method_cached(cls):
                cls.x += 1
                return cls.x

        @register(PyramidMixin)  # noqa
        class MTest:

            y = 0

            if inheritcache:
                @Declarations.classmethod_cache()
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(MTest, cls).method_cached()
            else:
                @classmethod
                def method_cached(cls):
                    cls.y += 2
                    return cls.y + super(MTest, cls).method_cached()

        @register(PyramidHTTP)
        class Test(PyramidMixin.MTest):
            pass

        self.add_controller()

    def test_model(self):
        registry = self.init_registry(self.add_model_with_method_cached)
        self.check_method_cached(registry)

    def test_model2(self):
        registry = self.init_registry(self.add_model_with_method_cached,
                                      inheritcache=True)
        self.check_inherited_method_cached(registry)

    def test_core(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_by_core)
        self.check_method_cached(registry)

    def test_core2(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_by_core, inheritcache=True)
        self.check_inherited_method_cached(registry)

    def test_mixin(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_by_mixin)
        self.check_method_cached(registry)

    def test_mixin2(self):
        registry = self.init_registry(
            self.add_model_with_method_cached_by_mixin, inheritcache=True)
        self.check_inherited_method_cached(registry)



from anyblok_pyramid.tests.testcase import PyramidDBTestCase
from anyblok import Declarations
register = Declarations.register
PyramidXmlRPC = Declarations.PyramidXmlRPC
PyramidMixin = Declarations.PyramidMixin
Core = Declarations.Core


class TestDeclarationPyramidXmlRPC(PyramidDBTestCase):

    def check_controller(self):
        res = self.xmlrpc('/test', 'methodA', params=(2, 3))
        self.assertEqual(res, [4, 6])
        res = self.xmlrpc('/test', 'methodB', params=(2, 3))
        self.assertEqual(res, [6, 9])

    def test_xmlrpc(self):
        def add_xmlrpc_contoller():
            @register(PyramidXmlRPC)
            class Test:

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return [x * 2 for x in args]

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_xmlrpc_without_route(self):
        def add_xmlrpc_contoller():

            @register(PyramidXmlRPC)
            class Test:

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return [x * 2 for x in args]

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

        with self.assertRaises(Declarations.Exception.PyramidException):
            self.init_registry(add_xmlrpc_contoller)

    def test_simple_subclass_controller(self):
        def add_xmlrpc_contoller():

            @register(PyramidXmlRPC)
            class Test:

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidXmlRPC)  # noqa
            class Test:

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_simple_subclass_base_xmlrpc(self):
        def add_xmlrpc_contoller():

            @register(Core)
            class PyramidBaseXmlRPC:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidXmlRPC)
            class Test:

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_simple_subclass_base_rpc(self):
        def add_xmlrpc_contoller():

            @register(Core)
            class PyramidBaseRPC:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidXmlRPC)
            class Test:

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_mixin_one_controller(self):
        def add_xmlrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidXmlRPC)
            class Test(PyramidMixin.PyMixin):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_mixin_two_controller(self):
        def add_xmlrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidXmlRPC)
            class Test(PyramidMixin.PyMixin):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            @register(PyramidXmlRPC)
            class Test2(PyramidMixin.PyMixin):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test2, self).methodA(*args)

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')
            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test2, '/test2')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()
        res = self.xmlrpc('/test2', 'methodA', params=(2, 3))
        self.assertEqual(res, [4, 6])

    def test_mixin_one_controller_with_subclass(self):
        def add_xmlrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidXmlRPC)
            class Test(PyramidMixin.PyMixin):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

            def inherit_Test():
                # put in the function else python take only on the first
                # Test class, because the fingerprint are the same
                @register(PyramidXmlRPC)
                class Test:

                    @PyramidXmlRPC.rpc_method(method='methodB')
                    def method_B(self, *args):
                        return [x * 3 for x in args]

            inherit_Test()

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_mixin_one_controller_by_subclass(self):
        def add_xmlrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidXmlRPC)
            class Test:

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            @register(PyramidXmlRPC)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_mixin_one_controller_by_subclass_and_with(self):
        def add_xmlrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidXmlRPC)
            class Test:

                def method_B(self, *args):
                    return [x * 3 for x in args]

            @register(PyramidXmlRPC)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

            def inherit_Test():
                # put in the function else python take only on the first
                # Test class, because the fingerprint are the same
                @register(PyramidXmlRPC)
                class Test:

                    @PyramidXmlRPC.rpc_method(method='methodB')
                    def method_B(self, *args):
                        return super(Test, self).method_B(*args)

            inherit_Test()

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_mixin_one_controller_with_subclass_and_subclass_mixin(self):
        def add_xmlrpc_contoller():

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, *args):
                    return None

            @register(PyramidXmlRPC)
            class Test:

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            @register(PyramidXmlRPC)  # noqa
            class Test(PyramidMixin.PyMixin):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

            @register(PyramidMixin)  # noqa
            class PyMixin:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_inherit_by_another_controller(self):
        def add_xmlrpc_contoller():

            @register(PyramidXmlRPC)
            class Test2:

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return [x * 2 for x in args]

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            @register(PyramidXmlRPC)
            class Test(PyramidXmlRPC.Test2):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return super(Test, self).method_B(*args)

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')
            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test2, '/test2')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_inherit_by_another_controller_and_subclass_maincontroller(self):
        def add_xmlrpc_contoller():

            @register(PyramidXmlRPC)
            class Test2:

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            @register(PyramidXmlRPC)
            class Test(PyramidXmlRPC.Test2):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return super(Test, self).method_B(*args)

            @register(PyramidXmlRPC)  # noqa
            class Test2:

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return [x * 2 for x in args]

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')
            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test2, '/test2')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()

    def test_inherit_core_and_mixin(self):
        def add_xmlrpc_contoller():

            @register(Core)
            class PyramidBaseRPC:

                def methodA(self, *args):
                    return [x * 2 for x in args]

            @register(PyramidMixin)
            class PyMixin:

                def methodA(self, *args):
                    return super(PyMixin, self).methodA(*args)

            @register(PyramidXmlRPC)
            class Test(PyramidMixin.PyMixin):

                @PyramidXmlRPC.rpc_method()
                def methodA(self, *args):
                    return super(Test, self).methodA(*args)

                @PyramidXmlRPC.rpc_method(method='methodB')
                def method_B(self, *args):
                    return [x * 3 for x in args]

            Declarations.PyramidXmlRPC.add_route(
                Declarations.PyramidXmlRPC.Test, '/test')

        self.init_registry(add_xmlrpc_contoller)
        self.check_controller()



from anyblok._argsparse import ArgsParseManager
from anyblok.registry import RegistryManager
from anyblok.tests.testcase import DBTestCase, BlokTestCase
from webtest import TestApp
from anyblok_pyramid.config import make_config
from pyramid.response import Response
import json
from pyramid_rpc.compat import xmlrpclib


class PyramidTestCase:

    @classmethod
    def setUpClass(cls):
        super(PyramidTestCase, cls).setUpClass()
        RegistryManager.add_needed_bloks('pyramid')

    def http(self, path, params=None, method='post'):
        resp = getattr(self.webserver, method)(path, params)
        self.assertEqual(resp.status_int, 200)
        return resp

    def jsonrpc(self, path, method, params=None):
        body = {
            'id': 5,
            'jsonrpc': '2.0',
            'method': method,
        }
        if params is not None:
            body['params'] = params

        kwargs = dict(content_type='application/json', params=json.dumps(body))
        resp = self.webserver.post(path, **kwargs)
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, 'application/json')
        result = resp.json
        self.assertEqual(result['jsonrpc'], '2.0')
        self.assertEqual(result['id'], 5)
        return result

    def xmlrpc(self, path, method, params=None):
        if params is None:
            params = tuple()

        xml = xmlrpclib.dumps(params, methodname=method).encode('utf-8')
        resp = self.webserver.post(path, content_type='text/xml', params=xml)
        self.assertEqual(resp.status_int, 200)
        self.assertEqual(resp.content_type, 'text/xml')
        return xmlrpclib.loads(resp.body)[0][0]

    def init_web_server(self):
        config = make_config()
        app = config.make_wsgi_app()
        self.webserver = TestApp(app)
        dbname = ArgsParseManager.get('dbname')
        self.webserver.post('/pyramid/testcase/database?database=%s' % dbname)


class PyramidDBTestCase(PyramidTestCase, DBTestCase):

    def init_registry(self, function, **kwargs):
        from anyblok import Declarations
        pyramid_routes = [] + Declarations.Pyramid.routes
        pyramid_views = [] + Declarations.Pyramid.views
        pyramid_http_routes = [] + Declarations.PyramidHTTP.routes
        pyramid_http_views = Declarations.PyramidHTTP.views.copy()
        pyramid_jsonrpc_routes = [] + Declarations.PyramidJsonRPC.routes
        pyramid_jsonrpc_methods = Declarations.PyramidJsonRPC.methods.copy()
        pyramid_xmlrpc_routes = [] + Declarations.PyramidXmlRPC.routes
        pyramid_xmlrpc_methods = Declarations.PyramidXmlRPC.methods.copy()

        def wrap_fnct(**wfkwargs):
            Declarations.Pyramid.add_route(
                'pyramidtestcasedatabase', '/pyramid/testcase/database')

            @Declarations.Pyramid.add_view('pyramidtestcasedatabase',
                                           request_method='POST')
            def save_db_in_session(request, database=None):
                request.session['database'] = database
                request.session.save()
                return Response('ok')

            function(**wfkwargs)

        try:
            res = super(PyramidDBTestCase, self).init_registry(wrap_fnct,
                                                               **kwargs)
            self.init_web_server()
        finally:
            Declarations.Pyramid.routes = pyramid_routes
            Declarations.Pyramid.views = pyramid_views
            Declarations.PyramidHTTP.routes = pyramid_http_routes
            Declarations.PyramidHTTP.views = pyramid_http_views
            Declarations.PyramidJsonRPC.routes = pyramid_jsonrpc_routes
            Declarations.PyramidJsonRPC.methods = pyramid_jsonrpc_methods
            Declarations.PyramidXmlRPC.routes = pyramid_xmlrpc_routes
            Declarations.PyramidXmlRPC.methods = pyramid_xmlrpc_methods

        return res


class PyramidBlokTestCase(PyramidTestCase, BlokTestCase):

    def setUp(self):
        super(PyramidBlokTestCase, self).setUp()
        self.init_web_server()
